# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import sys

from prettyqt import core, widgets, gui


class ColorChooserButton(widgets.Widget):

    color_updated = core.Signal(gui.Color)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.color = None
        layout = widgets.BoxLayout("horizontal", self)
        layout.set_margin(0)
        self.lineedit = widgets.LineEdit(self)
        self.lineedit.set_regex_validator(r"^#(?:[0-9a-fA-F]{6})$")
        layout.addWidget(self.lineedit)
        action = widgets.Action()
        action.triggered.connect(self.choose_color)
        self.button = widgets.ToolButton(self)
        self.button.setDefaultAction(action)
        layout.addWidget(self.button)

    def __getstate__(self):
        return dict(color=self.color,
                    enabled=self.isEnabled())

    def __setstate__(self, state):
        self.__init__()
        if state["color"]:
            self.set_color(state["color"])
        self.setEnabled(state["enabled"])

    @core.Slot()
    def choose_color(self):
        dlg = widgets.ColorDialog()
        if self.color:
            dlg.setCurrentColor(self.color)

        if dlg.exec_():
            self.set_color(dlg.current_color())
            self.color_updated.emit(dlg.current_color())

    def set_color(self, color):
        if isinstance(color, str):
            self.color = gui.Color(color)
        else:
            self.color = color
        self.lineedit.setText(self.color.name().upper())
        self.button.setStyleSheet(f"background-color: {self.color.name()};")


if __name__ == "__main__":
    app = widgets.Application(sys.argv)
    btn = ColorChooserButton()
    btn.set_color(gui.Color("green"))
    btn.show()
    btn.color_updated.connect(print)
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import widgets, gui


class ColorDialog(QtWidgets.QColorDialog):

    @classmethod
    def get_color(cls, preset=None, allow_alpha=False, parent=None):
        if isinstance(preset, str):
            preset = gui.Color(preset)
        if preset is None:
            preset = gui.Color()
        kwargs = dict(options=cls.ShowAlphaChannel) if allow_alpha else dict()
        return gui.Color(cls.getColor(preset, parent, **kwargs))

    def current_color(self):
        return gui.Color(self.currentColor())


if __name__ == "__main__":
    app = widgets.Application.create_default_app()
    ColorDialog.get_color()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import widgets


class SpinBox(QtWidgets.QSpinBox):

    def __getstate__(self):
        return dict(range=(self.minimum(), self.maximum()),
                    value=self.value(),
                    enabled=self.isEnabled(),
                    single_step=self.singleStep())

    def __setstate__(self, state):
        super().__init__()
        self.setRange(*state["range"])
        self.setValue(state["value"])
        self.setEnabled(state["enabled"])
        self.setSingleStep(state["single_step"])

    def set_enabled(self):
        self.setEnabled(True)

    def set_disabled(self):
        self.setEnabled(False)


if __name__ == "__main__":
    app = widgets.Application.create_default_app()
    widget = SpinBox()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import gui, widgets


class LineEdit(QtWidgets.QLineEdit):

    def __getstate__(self):
        return dict(text=self.text(),
                    enabled=self.isEnabled(),
                    font=gui.Font(self.font()),
                    validator=self.validator())

    def __setstate__(self, state):
        super().__init__()
        self.set_text(state["text"])
        self.setEnabled(state["enabled"])
        self.setFont(state["font"])
        self.setValidator(state["validator"])

    def font(self):
        return gui.Font(super().font())

    def set_enabled(self):
        self.setEnabled(True)

    def set_disabled(self):
        self.setEnabled(False)

    def set_font(self,
                 font_name: str,
                 font_size: int = -1,
                 weight: int = -1,
                 italic: bool = False):
        font = gui.Font(font_name, font_size, weight, italic)
        self.setFont(font)

    def append_text(self, text: str):
        self.set_text(self.text() + text)

    def set_text(self, text: str):
        self.setText(text)

    def set_read_only(self, value: bool = True):
        """set test to read only

        Args:
            value: True, for read-only, otherwise False
        """
        self.setReadOnly(value)

    def set_regex_validator(self, regex: str) -> gui.RegExpValidator:
        validator = gui.RegExpValidator(self)
        validator.set_regex(regex)
        self.setValidator(validator)
        return validator

    def set_color(self, color):
        self.setStyleSheet(f"background-color: {color};")

    def get_value(self):
        return self.text()


if __name__ == "__main__":
    app = widgets.Application.create_default_app()
    widget = LineEdit("This is a test")
    widget.set_regex_validator("[0-9]+")
    widget.setFont(gui.Font("Consolas"))
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets
import qtawesome as qta

from prettyqt import widgets


class ToolBox(QtWidgets.QToolBox):

    def __getitem__(self, index):
        return self.widget(index)

    def __iter__(self):
        return iter(self[i] for i in range(self.count()))

    def add_widget(self, widget, title=None, icon=None):
        if title is None:
            title = widget.objectName()
        if isinstance(icon, str):
            icon = qta.icon(icon)
        if icon:
            self.addItem(widget, icon, title)
        else:
            self.addItem(widget, title)


if __name__ == "__main__":
    app = widgets.Application.create_default_app()
    w = widgets.Widget()
    w2 = widgets.Widget()
    w2.setObjectName("objectName")
    tb = ToolBox()
    tb.add_widget(w, "title")
    tb.add_widget(w2)
    print(len(tb))
    tb.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore, QtWidgets


class Slider(QtWidgets.QSlider):

    def __getstate__(self):
        return dict(range=(self.minimum(), self.maximum()),
                    value=self.value(),
                    enabled=self.isEnabled(),
                    single_step=self.singleStep())

    def __setstate__(self, state):
        super().__init__()
        self.setRange(*state["range"])
        self.setValue(state["value"])
        self.setEnabled(state["enabled"])
        self.setSingleStep(state["single_step"])

    def is_horizontal(self) -> bool:
        """check if silder is horizontal

        Returns:
            True if horizontal, else False
        """
        return self.orientation() == QtCore.Qt.Horizontal

    def is_vertical(self) -> bool:
        """check if silder is vertical

        Returns:
            True if vertical, else False
        """
        return self.orientation() == QtCore.Qt.Vertical

    def set_horizontal(self):
        """set slider orientation to horizontal
        """
        self.setOrientation(QtCore.Qt.Horizontal)

    def set_vertical(self):
        """set slider orientation to vertical
        """
        self.setOrientation(QtCore.Qt.Vertical)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore, QtWidgets
from prettyqt import widgets


class FlowLayout(QtWidgets.QLayout):
    def __init__(self, parent=None, margin=0, spacing=-1):
        super().__init__(parent)

        if parent is not None:
            self.setMargin(margin)

        self.setSpacing(spacing)

        self.items = []

    def __getitem__(self, index):
        return self.itemAt(index)

    def __iter__(self):
        return iter(self[i] for i in range(self.count()))

    def __del__(self):
        item = self.takeAt(0)
        while item:
            item = self.takeAt(0)

    def addItem(self, item):
        self.items.append(item)

    def count(self):
        return len(self.items)

    def itemAt(self, index):
        if 0 <= index < len(self.items):
            return self.items[index]

        return None

    def takeAt(self, index):
        if 0 <= index < len(self.items):
            return self.items.pop(index)

        return None

    def expandingDirections(self):
        return QtCore.Qt.Orientations(QtCore.Qt.Orientation(0))

    def hasHeightForWidth(self):
        return True

    def heightForWidth(self, width):
        height = self.doLayout(QtCore.QRect(0, 0, width, 0), True)
        return height

    def setGeometry(self, rect):
        super(FlowLayout, self).setGeometry(rect)
        self.doLayout(rect, False)

    def sizeHint(self):
        return self.minimumSize()

    def minimumSize(self):
        size = QtCore.QSize()

        for item in self.items:
            size = size.expandedTo(item.minimumSize())

        size += QtCore.QSize(2 * self.contentsMargins().top(),
                             2 * self.contentsMargins().top())
        return size

    def doLayout(self, rect, testOnly):
        x = rect.x()
        y = rect.y()
        line_height = 0

        for item in self.items:
            wid = item.widget()
            pb = widgets.SizePolicy.PushButton
            spaceX = self.spacing() + wid.style().layoutSpacing(pb,
                                                                pb,
                                                                QtCore.Qt.Horizontal)
            spaceY = self.spacing() + wid.style().layoutSpacing(pb,
                                                                pb,
                                                                QtCore.Qt.Vertical)
            next_x = x + item.sizeHint().width() + spaceX
            if next_x - spaceX > rect.right() and line_height > 0:
                x = rect.x()
                y = y + line_height + spaceY
                next_x = x + item.sizeHint().width() + spaceX
                line_height = 0

            if not testOnly:
                item.setGeometry(QtCore.QRect(QtCore.QPoint(x, y), item.sizeHint()))

            x = next_x
            line_height = max(line_height, item.sizeHint().height())

        return y + line_height - rect.y()


if __name__ == '__main__':

    import sys

    app = QtWidgets.QApplication(sys.argv)
    widget = QtWidgets.QWidget()
    layout = FlowLayout()
    layout.addWidget(QtWidgets.QPushButton("Short"))
    layout.addWidget(QtWidgets.QPushButton("Longer"))
    layout.addWidget(QtWidgets.QPushButton("Different text"))
    layout.addWidget(QtWidgets.QPushButton("More text"))
    layout.addWidget(QtWidgets.QPushButton("Even longer button text"))
    widget.setLayout(layout)
    widget.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import qtawesome as qta
from qtpy import QtWidgets


class Action(QtWidgets.QAction):

    def __getstate__(self):
        return dict(text=self.text(),
                    enabled=self.isEnabled(),
                    shortcut=self.shortcut(),
                    tooltip=self.toolTip())

    def __setstate__(self, state):
        self.__init__()
        self.setText(state["text"])
        self.setEnabled(state["enabled"])
        self.set_shortcut(state["shortcut"])
        self.setToolTip(state["tooltip"])

    def set_enabled(self):
        self.setEnabled(True)

    def set_disabled(self):
        self.setEnabled(False)

    def set_tooltip(self, text):
        self.setToolTip(text)

    def set_icon(self, icon):
        if isinstance(icon, str):
            icon = qta.icon(icon)
        if icon:
            self.setIcon(icon)

    def set_shortcut(self, shortcut):
        if shortcut:
            self.setShortcut(shortcut)
